{% extends "base.html" %}

{% block extrahead %}
  {{ super() }}
  <!-- Load ome-explorer web component as ES module -->
  <script type="module" src="{{ 'javascripts/ome_explorer.js' | url }}"></script>
{% endblock %}
